@@ -23,7 +23,6 @@ import re
 import shutil
 import six
 import tempfile
-import time
 import yaml
 
 from heatclient.common import template_utils
@@ -59,8 +58,6 @@ class DeployOvercloud(command.Command):
 
         stack_is_new = stack is None
 
-        timestamp = int(time.time())
-        parameters['DeployIdentifier'] = timestamp
         parameters['UpdateIdentifier'] = ''
         parameters['StackAction'] = 'CREATE' if stack_is_new else 'UPDATE'
 
@@ -116,15 +113,6 @@ class DeployOvercloud(command.Command):
                 else:
                     parameters[param] = getattr(args, arg)
 
-        # Scaling needs extra parameters
-        number_controllers = int(parameters.get('ControllerCount', 0))
-        dhcp_agents_per_network = (min(number_controllers, 3) if
-                                   number_controllers else 1)
-
-        parameters.update({
-            'NeutronDhcpAgentsPerNetwork': dhcp_agents_per_network,
-        })
-
         return parameters
 
     def _create_registration_env(self, args):
@@ -211,7 +199,8 @@ class DeployOvercloud(command.Command):
         return env_files, localenv
 
     def _heat_deploy(self, stack, stack_name, template_path, parameters,
-                     env_files, timeout, tht_root, env, update_plan_only):
+                     env_files, timeout, tht_root, env, update_plan_only,
+                     skip_deploy_identifier):
         """Verify the Baremetal nodes are available and do a stack update"""
 
         clients = self.app.client_manager
@@ -258,8 +247,11 @@ class DeployOvercloud(command.Command):
             workflow_client)
 
         if not update_plan_only:
-            deployment.deploy_and_wait(self.log, clients, stack, stack_name,
-                                       self.app_args.verbose_level, timeout)
+            deployment.deploy_and_wait(
+                self.log, clients, stack, stack_name,
+                self.app_args.verbose_level,
+                timeout=timeout,
+                skip_deploy_identifier=skip_deploy_identifier)
 
     def _load_environment_directories(self, directories):
         if os.environ.get('TRIPLEO_ENVIRONMENT_DIRECTORY'):
@@ -471,17 +463,20 @@ class DeployOvercloud(command.Command):
 
         self._try_overcloud_deploy_with_compat_yaml(
             tht_root, stack, parsed_args.stack, parameters, env_files,
-            parsed_args.timeout, env, parsed_args.update_plan_only)
+            parsed_args.timeout, env, parsed_args.update_plan_only,
+            parsed_args.skip_deploy_identifier)
 
     def _try_overcloud_deploy_with_compat_yaml(self, tht_root, stack,
                                                stack_name, parameters,
                                                env_files, timeout,
-                                               env, update_plan_only):
+                                               env, update_plan_only,
+                                               skip_deploy_identifier):
         overcloud_yaml = os.path.join(tht_root, constants.OVERCLOUD_YAML_NAME)
         try:
             self._heat_deploy(stack, stack_name, overcloud_yaml,
                               parameters, env_files, timeout,
-                              tht_root, env, update_plan_only)
+                              tht_root, env, update_plan_only,
+                              skip_deploy_identifier)
         except ClientException as e:
             messages = 'Failed to deploy: %s' % str(e)
             raise ValueError(messages)
@@ -690,6 +685,16 @@ class DeployOvercloud(command.Command):
         network_type = parsed_args.neutron_network_type
         tunnel_types = parsed_args.neutron_tunnel_types
         tunnel_disabled = parsed_args.neutron_disable_tunneling
+
+        # We are no longer able to disable tunneling via a heat parameter
+        # so the client must set an empty tunnel types value for
+        # NeutronTunnelTypes to get the desired behavior.
+        if tunnel_disabled:
+            if tunnel_types:
+                raise oscexc.CommandError("Cannot specify tunnel types when "
+                                          "disabling tunneling")
+            parsed_args.neutron_tunnel_types = ''
+
         neutron_network_vlan_ranges = parsed_args.neutron_network_vlan_ranges
         if network_type == 'vlan' and not neutron_network_vlan_ranges:
             raise oscexc.CommandError(
@@ -1071,6 +1076,18 @@ class DeployOvercloud(command.Command):
             default=False,
             help=_('Force the overcloud post-deployment configuration.')
         )
+        parser.add_argument(
+            '--skip-deploy-identifier',
+            action='store_true',
+            default=False,
+            help=_('Skip generation of a unique identifier for the '
+                   'DeployIdentifier parameter. The software configuration '
+                   'deployment steps will only be triggered if there is an '
+                   'actual change to the configuration. This option should '
+                   'be used with Caution, and only if there is confidence '
+                   'that the software configuration does not need to be '
+                   'run, such as when scaling out certain roles.')
+        )
         reg_group = parser.add_argument_group('Registration Parameters')
         reg_group.add_argument(
             '--rhel-reg',
@@ -1140,13 +1157,13 @@ class DeployOvercloud(command.Command):
                 errors)
             if parsed_args.validation_warnings_fatal or \
                     parsed_args.validation_errors_fatal:
-                return
+                raise exceptions.InvalidConfiguration()
         if warnings > 0:
             self.log.error(
                 "Configuration has %d warnings, fix them before proceeding. ",
                 warnings)
             if parsed_args.validation_warnings_fatal:
-                return
+                raise exceptions.InvalidConfiguration()
         else:
             self.log.info("SUCCESS: No warnings or errors in deploy "
                           "configuration, proceeding.")
